public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
* [RFC PATCH] x86: make sure wakeup code is below 1M
@ 2009-11-09  8:51 Yinghai Lu
  2009-11-09 12:15 ` Rafael J. Wysocki
  0 siblings, 1 reply; 22+ messages in thread
From: Yinghai Lu @ 2009-11-09  8:51 UTC (permalink / raw)
  To: Ingo Molnar, Thomas Gleixner, H. Peter Anvin, Rafael J. Wysocki,
	Len Brown
  Cc: linux-kernel@vger.kernel.org


try to find_e820_area/reserve_early, and call acpi_reserve_memory early

to get area is below 1M

Signed-off-by: Yinghai Lu <yinghai@kernel.org>

---
 arch/x86/include/asm/acpi.h  |    2 +-
 arch/x86/kernel/acpi/sleep.c |   15 +++++++++------
 arch/x86/kernel/setup.c      |   13 +++++++------
 3 files changed, 17 insertions(+), 13 deletions(-)

Index: linux-2.6/arch/x86/include/asm/acpi.h
===================================================================
--- linux-2.6.orig/arch/x86/include/asm/acpi.h
+++ linux-2.6/arch/x86/include/asm/acpi.h
@@ -118,7 +118,7 @@ extern void acpi_restore_state_mem(void)
 extern unsigned long acpi_wakeup_address;
 
 /* early initialization routine */
-extern void acpi_reserve_bootmem(void);
+extern void acpi_reserve_memory(void);
 
 /*
  * Check if the CPU can handle C2 and deeper
Index: linux-2.6/arch/x86/kernel/setup.c
===================================================================
--- linux-2.6.orig/arch/x86/kernel/setup.c
+++ linux-2.6/arch/x86/kernel/setup.c
@@ -897,6 +897,13 @@ void __init setup_arch(char **cmdline_p)
 
 	reserve_brk();
 
+#ifdef CONFIG_ACPI_SLEEP
+	/*
+	 * Reserve low memory region for sleep support.
+	 * even before init_memory_mapping
+	 */
+	acpi_reserve_bootmem();
+#endif
 	init_gbpages();
 
 	/* max_pfn_mapped is updated here */
@@ -948,12 +955,6 @@ void __init setup_arch(char **cmdline_p)
 
 	initmem_init(0, max_pfn, acpi, k8);
 
-#ifdef CONFIG_ACPI_SLEEP
-	/*
-	 * Reserve low memory region for sleep support.
-	 */
-	acpi_reserve_bootmem();
-#endif
 	/*
 	 * Find and reserve possible boot-time SMP configuration:
 	 */

^ permalink raw reply	[flat|nested] 22+ messages in thread

end of thread, other threads:[~2009-11-13  8:12 UTC | newest]

Thread overview: 22+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
2009-11-09  8:51 [RFC PATCH] x86: make sure wakeup code is below 1M Yinghai Lu
2009-11-09 12:15 ` Rafael J. Wysocki
2009-11-11  2:27   ` [PATCH] x86: make sure wakeup code is below 1M -v2 Yinghai Lu
2009-11-11  7:48     ` Ingo Molnar
2009-11-11  7:57       ` H. Peter Anvin
2009-11-11  9:56       ` Rafael J. Wysocki
2009-11-11  9:12     ` ykzhao
2009-11-11 19:05       ` Yinghai Lu
2009-11-12  1:17         ` ykzhao
2009-11-12  3:12           ` Yinghai Lu
2009-11-12  3:37             ` ykzhao
2009-11-12  5:21               ` Yinghai Lu
2009-11-12  7:14                 ` ykzhao
2009-11-12  7:20                   ` H. Peter Anvin
2009-11-12  7:43                 ` ykzhao
2009-11-11 20:30     ` [tip:x86/mm] x86: Make sure wakeup trampoline code is below 1MB tip-bot for Yinghai Lu
2009-11-12  7:36     ` [PATCH] x86: make sure wakeup code is below 1M -v2 Pavel Machek
2009-11-12 19:25       ` Yinghai Lu
2009-11-12 19:32       ` Rafael J. Wysocki
2009-11-13  7:36         ` Ingo Molnar
2009-11-13  8:04           ` H. Peter Anvin
2009-11-13  8:12             ` Ingo Molnar

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Index: linux-2.6/arch/x86/kernel/acpi/sleep.c
===================================================================
--- linux-2.6.orig/arch/x86/kernel/acpi/sleep.c
+++ linux-2.6/arch/x86/kernel/acpi/sleep.c
@@ -119,29 +119,32 @@ void acpi_restore_state_mem(void)
 
 
 /**
  * acpi_reserve_bootmem - do _very_ early ACPI initialisation
  *
  * We allocate a page from the first 1MB of memory for the wakeup
  * routine for when we come back from a sleep state. The
  * runtime allocator allows specification of <16MB pages, but not
  * <1MB pages.
  */
-void __init acpi_reserve_bootmem(void)
+void __init acpi_reserve_memory(void)
 {
+	unsigned long mem;
+
 	if ((&wakeup_code_end - &wakeup_code_start) > WAKEUP_SIZE) {
 		printk(KERN_ERR
 		       "ACPI: Wakeup code way too big, S3 disabled.\n");
 		return;
 	}
 
-	acpi_realmode = (unsigned long)alloc_bootmem_low(WAKEUP_SIZE);
+	mem = find_e820_area(0, 1<<20, WAKEUP_SIZE, PAGE_SIZE);
 
-	if (!acpi_realmode) {
+	if (mem == -1L) {
 		printk(KERN_ERR "ACPI: Cannot allocate lowmem, S3 disabled.\n");
 		return;
 	}
-
-	acpi_wakeup_address = virt_to_phys((void *)acpi_realmode);
+	acpi_realmode = (unsigned long) phys_to_virt(mem);
+	acpi_wakeup_address = mem;
+	reserve_early(mem, mem + WAKEUP_SIZE, "WAKEUP");
 }
 
 
@@ -119,29 +119,32 @@ void acpi_restore_state_mem(void)
 
 /**
- * acpi_reserve_bootmem - do _very_ early ACPI initialisation
+ * acpi_reserve_memory - do _very_ early ACPI initialisation
  *
  * We allocate a page from the first 1MB of memory for the wakeup